<?php 
use yii\helpers\Url;
$this->registerJs("
var cartid = '{$model->id}';
var CreateSO_URL = '" . Url::to(["/netsuitesoap/createso"]) . "';
var AddDeposit_URL = '" . Url::to(["/netsuitesoap/adddeposit"]) . "';
var LinkToNS_URL = '" . Url::to(["/netsuite/linktons"]) . "';
", \yii\web\View::POS_BEGIN);
$jsFile = Yii::$app->assetManager->publish('@PortalPeople/netsuite/assets/js/cart.js');
$this->registerJsFile($jsFile[1], ['depends' => [\yii\web\JqueryAsset::class]]);