<?php
namespace portalpeople\netsuite\assets;

use yii\web\AssetBundle;

class SettingsAsset extends AssetBundle
{
    public $sourcePath = '@PortalPeople/netsuite/assets';
    public $css = [
        'settings.css',
    ];
    public $js = [
        'settings.js',
    ];
    public $depends = [
        'yii\web\JqueryAsset',
    ];
}
